@@ -23,6 +23,71 @@
 
 #ifdef WIN32
 
+int mingw_poll(struct pollfd *fds, unsigned int nfds, int timo)
+{
+    struct timeval timeout, *toptr;
+    fd_set ifds, ofds, efds, *ip, *op;
+    int i, rc;
+
+    /* Set up the file-descriptor sets in ifds, ofds and efds. */
+    FD_ZERO(&ifds);
+    FD_ZERO(&ofds);
+    FD_ZERO(&efds);
+    for (i = 0, op = ip = 0; i < nfds; ++i) {
+	fds[i].revents = 0;
+	if(fds[i].events & (POLLIN|POLLPRI)) {
+		ip = &ifds;
+		FD_SET(fds[i].fd, ip);
+	}
+	if(fds[i].events & POLLOUT) {
+		op = &ofds;
+		FD_SET(fds[i].fd, op);
+	}
+	FD_SET(fds[i].fd, &efds);
+    } 
+
+    /* Set up the timeval structure for the timeout parameter */
+    if(timo < 0) {
+	toptr = 0;
+    } else {
+	toptr = &timeout;
+	timeout.tv_sec = timo / 1000;
+	timeout.tv_usec = (timo - timeout.tv_sec * 1000) * 1000;
+    }
+
+#ifdef DEBUG_POLL
+    printf("Entering select() sec=%ld usec=%ld ip=%lx op=%lx\n",
+           (long)timeout.tv_sec, (long)timeout.tv_usec, (long)ip, (long)op);
+#endif
+    rc = select(0, ip, op, &efds, toptr);
+#ifdef DEBUG_POLL
+    printf("Exiting select rc=%d\n", rc);
+#endif
+
+    if(rc <= 0)
+	return rc;
+
+    if(rc > 0) {
+        for (i = 0; i < nfds; ++i) {
+            int fd = fds[i].fd;
+    	if(fds[i].events & (POLLIN|POLLPRI) && FD_ISSET(fd, &ifds))
+    		fds[i].revents |= POLLIN;
+    	if(fds[i].events & POLLOUT && FD_ISSET(fd, &ofds))
+    		fds[i].revents |= POLLOUT;
+    	if(FD_ISSET(fd, &efds))
+    		/* Some error was detected ... should be some way to know. */
+    		fds[i].revents |= POLLHUP;
+#ifdef DEBUG_POLL
+        printf("%d %d %d revent = %x\n", 
+                FD_ISSET(fd, &ifds), FD_ISSET(fd, &ofds), FD_ISSET(fd, &efds), 
+                fds[i].revents
+        );
+#endif
+        }
+    }
+    return rc;
+}
+
 #else
 #include <ifaddrs.h>
 #include <sys/un.h>
@@ -320,7 +385,6 @@ just_kill_connection:
     if (wsi->utf8_token[n].token)
       free (wsi->utf8_token[n].token);
 
-/*	fprintf(stderr, "closing fd=%d\n", wsi->sock); */
 
 #ifdef LWS_OPENSSL_SUPPORT
   if (wsi->ssl)
@@ -791,12 +855,12 @@ libwebsocket_service_fd (struct libwebsocket_context *context,
 
       /* Disable Nagle */
       opt = 1;
-#ifndef Darwin
 #ifdef Win32
       setsockopt (accept_fd,SOL_TCP,TCP_NODELAY,(const char *)&opt,sizeof(opt));
+#elif defined(Darwin)
+      setsockopt (accept_fd, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof (opt));
 #else
       setsockopt (accept_fd, SOL_TCP, TCP_NODELAY, &opt, sizeof (opt));
-#endif
 #endif
 
       if (context->fds_count >= MAX_CLIENTS)
@@ -1841,8 +1905,7 @@ libwebsocket_service_fd (struct libwebsocket_context *context,
       if (pollfd->revents & (POLLERR | POLLHUP))
         {
 
-          fprintf (stderr, "Session Socket %p (fd=%d) dead\n",
-                   (void *) wsi, pollfd->fd);
+          debug ("Session Socket %p (fd=%d) dead\n", (void *) wsi, pollfd->fd);
 
           libwebsocket_close_and_free_session (context, wsi,
                                                LWS_CLOSE_STATUS_NOSTATUS);
@@ -2629,12 +2692,12 @@ libwebsocket_create_context (int port, const char *interf,
 
       /* Disable Nagle */
       opt = 1;
-#ifndef Darwin
 #ifdef Win32
       setsockopt (sockfd,SOL_TCP,TCP_NODELAY,(const char *)&opt,sizeof(opt));
+#elif defined(Darwin)
+      setsockopt (sockfd, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof (opt));
 #else
       setsockopt (sockfd, SOL_TCP, TCP_NODELAY, &opt, sizeof (opt));
-#endif
 #endif
 
       bzero ((char *) &serv_addr, sizeof (serv_addr));
@@ -2916,11 +2979,11 @@ libwebsockets_broadcast (const struct libwebsocket_protocols *protocol,
 
       for (n = 0; n < FD_HASHTABLE_MODULUS; n++)
         {
-
+          if(!context) return 0;
           for (m = 0; m < context->fd_hashtable[n].length; m++)
             {
-
               wsi = context->fd_hashtable[n].wsi[m];
+              if(!wsi) continue;
 
               if (wsi->mode != LWS_CONNMODE_WS_SERVING)
                 continue;
