@@ -40,7 +40,23 @@
 #include <ws2tcpip.h>
 #include <Winsock2.h>
 #include <windows.h>
-#include "poll.h"
+/* winsock doesn't feature poll(), so there is a version implemented
+ * in terms of select() in mingw.c. The following definitions
+ * are copied from linux man pages. A poll() macro is defined to
+ * call the version in mingw.c.
+ */
+#define POLLIN      0x0001    /* There is data to read */
+#define POLLPRI     0x0002    /* There is urgent data to read */
+#define POLLOUT     0x0004    /* Writing now will not block */
+#define POLLERR     0x0008    /* Error condition */
+#define POLLHUP     0x0010    /* Hung up */
+#define POLLNVAL    0x0020    /* Invalid request: fd not open */
+struct pollfd {
+  SOCKET fd;        /* file descriptor */
+  short events;     /* requested events */
+  short revents;    /* returned events */
+};
+#define poll(x, y, z)        mingw_poll(x, y, z)
 
 #else
 
@@ -76,11 +92,6 @@
 #define USE_RINTERNALS
 #include <Rinternals.h>
 
-
-#if 0
-#define DEBUG
-#endif
-
 #ifdef DEBUG
 #ifdef WIN32
 static
